package com.amirouter.server.CLI;

import java.io.File;
import java.io.FilenameFilter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.function.Function;

import org.jline.keymap.BindingReader;
import org.jline.keymap.KeyMap;
import org.jline.terminal.Terminal;
import org.jline.utils.AttributedString;
import org.springframework.shell.component.support.AbstractTextComponent.TextComponentContext.MessageLevel;
import org.springframework.util.StringUtils;

/**
 * Shell input path customization
 */
public class CustomPathInput extends org.springframework.shell.component.PathInput {

	/**
	 * Constructor
	 * 
	 * @param terminal
	 */
	public CustomPathInput(Terminal terminal) {
		super(terminal);
	}

	/**
	 * Constructor
	 * 
	 * @param terminal
	 * @param name
	 * @param renderer
	 */
	public CustomPathInput(Terminal terminal, String name,
			Function<PathInputContext, List<AttributedString>> renderer) {
		super(terminal, name, renderer);
	}

	/**
	 * Constructor
	 * 
	 * @param terminal
	 * @param name
	 */
	public CustomPathInput(Terminal terminal, String name) {
		super(terminal, name);
	}

	@Override
	protected boolean read(BindingReader bindingReader, KeyMap<String> keyMap, PathInputContext context) {

		String operation = bindingReader.readBinding(keyMap);
		String input;
		switch (operation) {
		case OPERATION_CHAR:
			String lastBinding = bindingReader.getLastBinding();
			input = context.getInput();
			if (input == null) {
				input = lastBinding;
			} else {
				input = input + lastBinding;
			}
			context.setInput(input);
			checkPath(input, context, true);
			break;
		case OPERATION_BACKSPACE:
			input = context.getInput();
			if (StringUtils.hasLength(input)) {
				input = input.length() > 1 ? input.substring(0, input.length() - 1) : null;
			}
			context.setInput(input);
			checkPath(input, context, true);
			break;
		case OPERATION_EXIT:
			if (StringUtils.hasText(context.getInput())) {
				context.setResultValue(Paths.get(context.getInput()));
			}
			return true;
		default:
			break;
		}
		return false;
	}

	private void checkPath(String path, PathInputContext context, boolean isListContent) {
		if (!StringUtils.hasText(path)) {
			context.setMessage(null);
			return;
		}
		Path p = resolvePath(path);
		boolean isDirectory = Files.isDirectory(p);
		if (isDirectory) {

			context.setMessage(listContent(path), MessageLevel.INFO);

		} else {

			File f = p.toFile();

			if (f.exists()) {
				context.setMessage("Path ok", MessageLevel.INFO);
				return;
			}

			if (isListContent) {
				try {

					if (p.getParent() == null) {
						return;
					}

					if (!p.getParent().toFile().exists()) {
						return;
					}

					context.setMessage(listContentByPattern(p.getParent().toRealPath().toString(), f.getName()),
							MessageLevel.INFO);

				} catch (Exception e) {
					e.printStackTrace();
				}
			}

		}
	}

	private String listContent(String path) {
		String[] pathnames;

		File f = new File(path);

		pathnames = f.list();

		StringBuilder sb = new StringBuilder("\n");

		for (String pathname : pathnames) {
			sb.append(String.format("%s\n", pathname));
		}

		sb.append("\n");

		return sb.toString();
	}

	private String listContentByPattern(String path, String startPattern) {
		String[] pathnames;

		File f = new File(path);

		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File f, String name) {
				return name.startsWith(startPattern);
			}
		};

		pathnames = f.list(filter);

		StringBuilder sb = new StringBuilder("\n");

		for (String pathname : pathnames) {
			sb.append(String.format("%s\n", pathname));
		}

		sb.append("\n");

		return sb.toString();
	}

}
