package com.amirouter.server.asterisk;

import java.io.IOException;
import java.util.Iterator;
import java.util.concurrent.ConcurrentHashMap;

import org.asteriskjava.manager.AuthenticationFailedException;
import org.asteriskjava.manager.ManagerConnection;
import org.asteriskjava.manager.ManagerConnectionFactory;
import org.asteriskjava.manager.ManagerConnectionState;
import org.asteriskjava.manager.ManagerEventListener;
import org.asteriskjava.manager.TimeoutException;
import org.asteriskjava.manager.action.ManagerAction;
import org.asteriskjava.manager.action.StatusAction;
import org.asteriskjava.manager.event.ManagerEvent;
import org.asteriskjava.manager.response.ManagerResponse;

/**
 * Remote asterisk server proxy class
 *
 */
public class AsteriskProxy implements ManagerEventListener {
	private ManagerConnection managerConnection;

	private ConcurrentHashMap<Integer, AsteriskEvtHandler> remAsteriskEvtHndlerCat;
	private int clientNumber = 1;
	private final String proxyId;
	private AsteriskInfo info;

	/**
	 * Take a remote asterisk server connection informations
	 * 
	 * @param asteriskServerInfo
	 * @throws IOException
	 */
	public AsteriskProxy(AsteriskInfo asteriskServerInfo, AsteriskInfo info) throws IOException {

		this.info = info;

		remAsteriskEvtHndlerCat = new ConcurrentHashMap<>();

		proxyId = asteriskServerInfo.getHostname();

		ManagerConnectionFactory factory = new ManagerConnectionFactory(asteriskServerInfo.getHostname(),
				asteriskServerInfo.getPort(), asteriskServerInfo.getUsername(), asteriskServerInfo.getPassword());

		this.managerConnection = factory.createManagerConnection();
	}

	/**
	 * Pretty print messages with asterisk identifier as prefix
	 * 
	 * @param msg message to print
	 */
	public void printMsg(String msg) {
		System.out.println(String.format("--> [asterisk %s] %s", proxyId, msg));
	}

	/**
	 * Setup remote asterisk server connection and start listening for event
	 * 
	 * @throws IOException
	 * @throws AuthenticationFailedException
	 * @throws TimeoutException
	 * @throws InterruptedException
	 */
	public void run() throws IOException, AuthenticationFailedException, TimeoutException, InterruptedException {
		managerConnection.addEventListener(this);

		managerConnection.login();

		managerConnection.sendAction(new StatusAction());

	}

	/**
	 * Register an event handler
	 * 
	 * @param remAsteriskEvtHndler handler function to call on event
	 * @return The registration Id or -1 if error
	 */
	public int registerToEvent(AsteriskEvtHandler remAsteriskEvtHndler) {
		try {
			if (remAsteriskEvtHndler != null) {
				remAsteriskEvtHndlerCat.put(clientNumber, remAsteriskEvtHndler);
				printMsg("--> registered successfully clientID=" + clientNumber);
				return clientNumber++;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return -1;
	}

	/**
	 * Unregister an event handler using the registration Id
	 * 
	 * @param clientID the registration Id
	 * @return The registration Id or -1 if error
	 */
	public int unRegisterToEvent(int clientID) {
		try {
			if (remAsteriskEvtHndlerCat.containsKey(clientID)) {
				remAsteriskEvtHndlerCat.remove(clientID);
				printMsg("--> unregistered successfully clientID=" + clientID);
				return clientID;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return -1;
	}

	/**
	 * The event notification routine for all registered client on this specific
	 * asterisk server proxy
	 */
	public void onManagerEvent(ManagerEvent event) {

		if (remAsteriskEvtHndlerCat.size() > 0) {

			printMsg((String.format("--> starting new event notification (number of subscribed clients: %d)",
					remAsteriskEvtHndlerCat.size())));

			AsteriskEvtHandler hndlr;

			Iterator<ConcurrentHashMap.Entry<Integer, AsteriskEvtHandler>> itr = remAsteriskEvtHndlerCat.entrySet()
					.iterator();

			ConcurrentHashMap.Entry<Integer, AsteriskEvtHandler> entry;

			while (itr.hasNext()) {

				entry = itr.next();

				hndlr = entry.getValue();

				hndlr.onRemAsteriskEvt(event);

			}

		}
	}

	/**
	 * Exit by disconnecting from the proxied asterisk server
	 */
	public void finish() {
		try {

			managerConnection.removeEventListener(this);

			managerConnection.logoff();

		} catch (IllegalStateException e) {
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * Send an action to the connected asterisk server
	 * 
	 * @param action action to be sent. @see org.asteriskjava.manager.action
	 * @return the result
	 * @throws IllegalArgumentException
	 * @throws IllegalStateException
	 * @throws IOException
	 * @throws TimeoutException
	 */
	public ManagerResponse relayAction(ManagerAction action)
			throws IllegalArgumentException, IllegalStateException, IOException, TimeoutException {

		return managerConnection.sendAction(action);

	}

	/**
	 * Get the proxied asterisk server connection state.
	 * 
	 * @return The connection state. @see ManagerConnectionState
	 */
	public ManagerConnectionState getConnectionState() {
		return managerConnection.getState();
	}

	/**
	 * @return the info
	 */
	public AsteriskInfo getInfo() {
		return info;
	}
}
