package com.amirouter.server.services;

import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

import org.asteriskjava.manager.TimeoutException;
import org.asteriskjava.manager.action.ExtensionStateAction;
import org.asteriskjava.manager.action.ManagerAction;
import org.asteriskjava.manager.event.ManagerEvent;
import org.asteriskjava.manager.event.PeerStatusEvent;
import org.asteriskjava.manager.event.NewChannelEvent;
import org.asteriskjava.manager.event.NewStateEvent;
import org.asteriskjava.manager.event.ContactStatusEvent;
import org.asteriskjava.manager.event.HangupEvent;
import org.asteriskjava.manager.event.HangupRequestEvent;
import org.asteriskjava.manager.event.ParkedCallEvent;
import org.asteriskjava.manager.event.UnparkedCallEvent;
import org.asteriskjava.manager.event.ParkedCallTimeOutEvent;
import org.asteriskjava.manager.event.ParkedCallGiveUpEvent;
import org.asteriskjava.manager.event.DialBeginEvent;
import org.asteriskjava.manager.event.HoldEvent;
import org.asteriskjava.manager.event.UnholdEvent;
import org.asteriskjava.manager.response.ExtensionStateResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.amirouter.server.ami.AmiClientContext;
import com.amirouter.server.ami.AmiResponseBehavior;
import com.amirouter.server.ami.AmiResponseWrapper;
import com.amirouter.server.ami.AmiSpecs;
import com.amirouter.server.asterisk.AsteriskProxy;

/**
 * Handle all client ami requests
 */
@Service
public class AmiProtocolService {

	@Autowired
	private UserService userService;

	/**
	 * Client ami action handler
	 * 
	 * @param AmiCmd           client ami action command
	 * @param amiClientContext client context
	 * @return ami response wrapper. @see AmiResponseWrapper
	 */
	public AmiResponseWrapper handleAmiActionCmd(Map<String, String> AmiCmd, AmiClientContext amiClientContext) {

		switch (AmiCmd.getOrDefault("Action", "")) {
		case "Login":
			return LoginAction(AmiCmd, amiClientContext);

		case "Logoff":
			return LogoffAction(AmiCmd, amiClientContext);

		case "ExtensionState":
			return ExtensionStateAction(AmiCmd, amiClientContext);

		default:

			Map<String, String> amiResponse = new LinkedHashMap<String, String>();
			amiResponse.put("Response", "Error");
			amiResponse.put("Message", String.format("Invalid/unknown comand: %s", AmiCmd.getOrDefault("Action", "")));

			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiResponse);
		}

	}

	/**
	 * User (client) management service
	 * 
	 * @return user (client) management service
	 */
	public UserService getUserService() {
		return userService;
	}

	private AmiResponseWrapper LoginAction(Map<String, String> AmiCmd, AmiClientContext amiClientContext) {

		boolean isValid = AmiSpecs.validateSpec(AmiCmd, AmiSpecs.AMI_LOGIN_SPEC);

		if (!isValid) {

			System.err.println("receved an invalid LoginAction command");

			Map<String, String> errResp = new LinkedHashMap<String, String>();

			errResp.put("Response", "Error");
			errResp.put("Message", "Authentication failed");

			return new AmiResponseWrapper(AmiResponseBehavior.SEND_AND_DISCONNECT, errResp);

		}

		Map<String, String> amiResponse = new LinkedHashMap<String, String>();

		boolean signRes = userService.signIn(AmiCmd.get("Username"), AmiCmd.get("Secret"), amiClientContext);

		if (signRes) {

			amiResponse.put("Response", "Success");
			amiResponse.put("Message", "Authentication accepted");

			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiResponse);

		} else {

			amiResponse.put("Response", "Error");
			amiResponse.put("Message", "Authentication failed");

			return new AmiResponseWrapper(AmiResponseBehavior.SEND_AND_DISCONNECT, amiResponse);
		}

	}

	private AmiResponseWrapper LogoffAction(Map<String, String> AmiCmd, AmiClientContext amiClientContext) {

		Map<String, String> amiResponse = new LinkedHashMap<String, String>();

		amiResponse.put("Response", "Goodbye");
		amiResponse.put("Message", "Thank for all the fish.");

		return new AmiResponseWrapper(AmiResponseBehavior.SEND_AND_DISCONNECT, amiResponse);

	}

	private AmiResponseWrapper ExtensionStateAction(Map<String, String> AmiCmd, AmiClientContext amiClientContext) {

		boolean isValid = AmiSpecs.validateSpec(AmiCmd, AmiSpecs.AMI_EXTENSIONSTATE_SPEC);

		if ((!isValid) || (!userService.isSignedIn(amiClientContext))) {
			System.err.println(String.format("invalid or unauth. ExtensionStateAction command. isValid=%s", isValid));
			return new AmiResponseWrapper(AmiResponseBehavior.IGNORE, null);
		}

		try {

			String exten = AmiCmd.get("Exten");

			String context = AmiCmd.get("Context");

			boolean usesipregs = amiClientContext.getSrvConfig().isUsesipregs();

			AsteriskServerInfos asteriskServerInfos = userService.getAsteriskServerInfos(usesipregs, exten,
					amiClientContext);

			if (asteriskServerInfos == null) {
				System.err.println("failled to retreive asteriskServerInfos");
				return new AmiResponseWrapper(AmiResponseBehavior.IGNORE, null);
			}

			String tmpAstIp = amiClientContext.getAsteriskIpByNodeName(asteriskServerInfos.getRegserver());

			if (tmpAstIp == null) {
				System.err.println("failled to retreive asterisk hostname");
				return new AmiResponseWrapper(AmiResponseBehavior.IGNORE, null);
			}

			ManagerAction action = new ExtensionStateAction(asteriskServerInfos.getName(), context);

			AsteriskProxy tmpAstProxy = amiClientContext.getAsteriskProxyByHostName(tmpAstIp);

			ExtensionStateResponse actionResponse = (ExtensionStateResponse) tmpAstProxy.relayAction(action);

			if (actionResponse == null) {
				System.err.println("ManagerResponse (ExtensionStateResponse) is null");
				return new AmiResponseWrapper(AmiResponseBehavior.IGNORE, null);
			}

			String ActionID = AmiCmd.get("ActionID");

			Map<String, String> amiResponse = new LinkedHashMap<String, String>();

			amiResponse.put("Response", actionResponse.getResponse());

			if (ActionID != null) {
				amiResponse.put("ActionID", ActionID);
			}

			amiResponse.put("Message", actionResponse.getMessage());

			amiResponse.put("Exten", actionResponse.getExten());
			amiResponse.put("Context", actionResponse.getContext());
			amiResponse.put("Hint", actionResponse.getHint());
			amiResponse.put("Status", actionResponse.getStatus().toString());
			amiResponse.put("StatusText", actionResponse.getStatusText());

			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiResponse);

		} catch (IllegalArgumentException | IllegalStateException | IOException | TimeoutException e) {
			e.printStackTrace();
			return new AmiResponseWrapper(AmiResponseBehavior.IGNORE, null);
		}

	}

	/**
	 * Asterisk server ami event handler
	 * 
	 * @param event            received event from one the managed asterisk servers
	 * @param amiClientContext current dialoging client context
	 * @return ami response wrapper destined to the client. @see AmiResponseWrapper
	 */
	public AmiResponseWrapper handleAmiEvent(ManagerEvent event, AmiClientContext amiClientContext) {

		if (event instanceof PeerStatusEvent) {

			PeerStatusEvent evtObj = (PeerStatusEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "PeerStatus");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("ChannelType", evtObj.getChannelType());
			amiEvtResponse.put("Peer", evtObj.getPeer());
			amiEvtResponse.put("PeerStatus", evtObj.getPeerStatus());

			if (evtObj.getCause() != null) {
				amiEvtResponse.put("Cause", evtObj.getCause());
			}

			if (evtObj.getAddress() != null) {
				amiEvtResponse.put("Address", evtObj.getAddress());
			}

			if (evtObj.getPort() != null) {
				amiEvtResponse.put("Port", evtObj.getPort().toString());
			}

			if (evtObj.getTime() != null) {
				amiEvtResponse.put("Time", evtObj.getTime().toString());
			}

			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);
		}

		if (event instanceof ParkedCallEvent) {

			ParkedCallEvent evtObj = (ParkedCallEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "ParkedCall");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("ParkeeChannel", evtObj.getParkeeChannel());
			amiEvtResponse.put("ParkeeChannelState", evtObj.getParkeeChannelState().toString());
			amiEvtResponse.put("ParkeeChannelStateDesc", evtObj.getParkeeChannelStateDesc());
			amiEvtResponse.put("ParkeeCallerIDNum", evtObj.getParkeeCallerIDNum());
			amiEvtResponse.put("ParkeeCallerIDName", evtObj.getParkeeCallerIDName());
			amiEvtResponse.put("ParkeeConnectedLineNum", evtObj.getParkeeConnectedLineNum());
			amiEvtResponse.put("ParkeeConnectedLineName", evtObj.getParkeeConnectedLineName());
			amiEvtResponse.put("ParkeeLanguage", evtObj.getParkeeLanguage());
			amiEvtResponse.put("ParkeeAccountCode", evtObj.getParkeeAccountCode());
			amiEvtResponse.put("ParkeeContext", evtObj.getParkeeContext());
			amiEvtResponse.put("ParkeeExten", evtObj.getParkeeExten());
			amiEvtResponse.put("ParkeePriority", evtObj.getParkeePriority().toString());
			amiEvtResponse.put("ParkeeUniqueid", evtObj.getParkeeUniqueid());
			amiEvtResponse.put("ParkeeLinkedid", evtObj.getParkeelinkedid());			
			amiEvtResponse.put("ParkerDialString", evtObj.getParkerDialString());			
			amiEvtResponse.put("Parkinglot", evtObj.getParkinglot());
			amiEvtResponse.put("ParkingSpace", evtObj.getParkingSpace());
			amiEvtResponse.put("ParkingTimeout", evtObj.getParkingTimeout().toString());
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);

		}

		if (event instanceof ParkedCallGiveUpEvent) {

			ParkedCallGiveUpEvent evtObj = (ParkedCallGiveUpEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "ParkedCallGiveUp");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("ParkeeChannel", evtObj.getParkeeChannel());
			amiEvtResponse.put("ParkeeChannelState", evtObj.getParkeeChannelState().toString());
			amiEvtResponse.put("ParkeeChannelStateDesc", evtObj.getParkeeChannelStateDesc());
			amiEvtResponse.put("ParkeeCallerIDNum", evtObj.getParkeeCallerIDNum());
			amiEvtResponse.put("ParkeeCallerIDName", evtObj.getParkeeCallerIDName());
			amiEvtResponse.put("ParkeeConnectedLineNum", evtObj.getParkeeConnectedLineNum());
			amiEvtResponse.put("ParkeeConnectedLineName", evtObj.getParkeeConnectedLineName());
			amiEvtResponse.put("ParkeeLanguage", evtObj.getParkeeLanguage());
			amiEvtResponse.put("ParkeeAccountCode", evtObj.getParkeeAccountCode());
			amiEvtResponse.put("ParkeeContext", evtObj.getParkeeContext());
			amiEvtResponse.put("ParkeeExten", evtObj.getParkeeExten());
			amiEvtResponse.put("ParkeePriority", evtObj.getParkeePriority().toString());
			amiEvtResponse.put("ParkeeUniqueid", evtObj.getParkeeUniqueid());
			//			amiEvtResponse.put("ParkeeLinkedid", evtObj.getParkeelinkedid()); // Seems missing in the library

			amiEvtResponse.put("ParkerChannel", evtObj.getParkerChannel());
			amiEvtResponse.put("ParkerChannelState", evtObj.getParkerChannelState().toString());
			amiEvtResponse.put("ParkerChannelStateDesc", evtObj.getParkerChannelStateDesc());
			amiEvtResponse.put("ParkerCallerIDNum", evtObj.getParkerCallerIDNum());
			amiEvtResponse.put("ParkerCallerIDName", evtObj.getParkerCallerIDName());
			amiEvtResponse.put("ParkerConnectedLineNum", evtObj.getParkerConnectedLineNum());
			amiEvtResponse.put("ParkerConnectedLineName", evtObj.getParkerConnectedLineName());
			//			amiEvtResponse.put("ParkerLanguage", evtObj.getParkerLanguage()); // Seems missing in the library 
			amiEvtResponse.put("ParkerAccountCode", evtObj.getParkerAccountCode());
			amiEvtResponse.put("ParkerContext", evtObj.getParkerContext());
			amiEvtResponse.put("ParkerExten", evtObj.getParkerExten());
			amiEvtResponse.put("ParkerPriority", evtObj.getParkerPriority().toString());
			amiEvtResponse.put("ParkerUniqueid", evtObj.getParkerUniqueid());
			//			amiEvtResponse.put("ParkerLinkedid", evtObj.getParkerlinkedid()); // Seems missing in the library
			amiEvtResponse.put("ParkerDialString", evtObj.getParkerDialString());			

			amiEvtResponse.put("Parkinglot", evtObj.getParkinglot());
			amiEvtResponse.put("ParkingSpace", evtObj.getParkingSpace());
			amiEvtResponse.put("ParkingTimeout", evtObj.getParkingTimeout().toString());
			amiEvtResponse.put("ParkingDuration", evtObj.getParkingDuration().toString());
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);

		}

		if (event instanceof UnparkedCallEvent) {

			UnparkedCallEvent evtObj = (UnparkedCallEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "UnparkedCall");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("ParkeeChannel", evtObj.getParkeeChannel());
			amiEvtResponse.put("ParkeeChannelState", evtObj.getParkeeChannelState().toString());
			amiEvtResponse.put("ParkeeChannelStateDesc", evtObj.getParkeeChannelStateDesc());
			amiEvtResponse.put("ParkeeCallerIDNum", evtObj.getParkeeCallerIDNum());
			amiEvtResponse.put("ParkeeCallerIDName", evtObj.getParkeeCallerIDName());
			amiEvtResponse.put("ParkeeConnectedLineNum", evtObj.getParkeeConnectedLineNum());
			amiEvtResponse.put("ParkeeConnectedLineName", evtObj.getParkeeConnectedLineName());
			amiEvtResponse.put("ParkeeLanguage", evtObj.getParkeeLanguage());
			amiEvtResponse.put("ParkeeAccountCode", evtObj.getParkeeAccountCode());
			amiEvtResponse.put("ParkeeContext", evtObj.getParkeeContext());
			amiEvtResponse.put("ParkeeExten", evtObj.getParkeeExten());
			amiEvtResponse.put("ParkeePriority", evtObj.getParkeePriority().toString());
			amiEvtResponse.put("ParkeeUniqueid", evtObj.getParkeeUniqueid());
			//			amiEvtResponse.put("ParkeeLinkedid", evtObj.getParkeelinkedid()); // Seems missing in the library
			amiEvtResponse.put("ParkerChannel", evtObj.getParkerChannel());			
			if (evtObj.getParkerChannelState() != null) {
			    amiEvtResponse.put("ParkerChannelState", evtObj.getParkerChannelState().toString());
			}
			amiEvtResponse.put("ParkerChannelStateDesc", evtObj.getParkerChannelStateDesc());
			amiEvtResponse.put("ParkerCallerIDNum", evtObj.getParkerCallerIDNum());
			amiEvtResponse.put("ParkerCallerIDName", evtObj.getParkerCallerIDName());
			amiEvtResponse.put("ParkerConnectedLineNum", evtObj.getParkerConnectedLineNum());
			amiEvtResponse.put("ParkerConnectedLineName", evtObj.getParkerConnectedLineName());
			//			amiEvtResponse.put("ParkerLanguage", evtObj.getParkerLanguage()); // Seems missing in the library 
			amiEvtResponse.put("ParkerAccountCode", evtObj.getParkerAccountCode());
			amiEvtResponse.put("ParkerContext", evtObj.getParkerContext());
			amiEvtResponse.put("ParkerExten", evtObj.getParkerExten());
			amiEvtResponse.put("ParkerPriority", evtObj.getParkerPriority());
			amiEvtResponse.put("ParkerUniqueid", evtObj.getParkerUniqueid());
			//			amiEvtResponse.put("ParkerLinkedid", evtObj.getParkerlinkedid()); // Seems missing in the library
			amiEvtResponse.put("ParkerDialString", evtObj.getParkerDialString());			

			amiEvtResponse.put("Parkinglot", evtObj.getParkinglot());
			amiEvtResponse.put("ParkingSpace", evtObj.getParkingSpace());
			amiEvtResponse.put("ParkingTimeout", evtObj.getParkingTimeout().toString());
			amiEvtResponse.put("ParkingDuration", evtObj.getParkingDuration().toString());
			amiEvtResponse.put("RetrieverChannel", evtObj.getRetrieverChannel());
			amiEvtResponse.put("RetrieverChannelState", evtObj.getRetrieverChannelState().toString());
			amiEvtResponse.put("RetrieverChannelStateDesc", evtObj.getRetrieverChannelStateDesc());
			amiEvtResponse.put("RetrieverCallerIDNum", evtObj.getRetrieverCallerIDNum());
			amiEvtResponse.put("RetrieverCallerIDName", evtObj.getRetrieverCallerIDName());
			amiEvtResponse.put("RetrieverConnectedLineNum", evtObj.getRetrieverConnectedLineNum());
			amiEvtResponse.put("RetrieverConnectedLineName", evtObj.getRetrieverConnectedLineName());
			//			amiEvtResponse.put("RetrieverLanguage", evtObj.getRetrieverLanguage()); // Seems missing in the library
			amiEvtResponse.put("RetrieverAccountCode", evtObj.getRetrieverAccountCode());
			amiEvtResponse.put("RetrieverContext", evtObj.getRetrieverContext());
			amiEvtResponse.put("RetrieverExten", evtObj.getRetrieverExten());
			amiEvtResponse.put("RetrieverPriority", evtObj.getRetrieverPriority());
			amiEvtResponse.put("RetrieverUniqueid", evtObj.getRetrieverUniqueid());
			//			amiEvtResponse.put("RetrieverLinkedid", evtObj.getRetrieverLinkedid()); // Seems missing in the library
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);

		}

		if (event instanceof ContactStatusEvent) {

			ContactStatusEvent evtObj = (ContactStatusEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "ContactStatus");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("URI", evtObj.getUri());
			amiEvtResponse.put("ContactStatus", evtObj.getContactStatus());
			amiEvtResponse.put("AOR", evtObj.getAor());
			amiEvtResponse.put("EndpointName", evtObj.getEndpointName());
			amiEvtResponse.put("RoundtripUsec", evtObj.getRoundtripUsec());
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);		    

		}

		if (event instanceof HangupEvent) {

			HangupEvent evtObj = (HangupEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "Hangup");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("Channel", evtObj.getChannel());
			amiEvtResponse.put("ChannelState", evtObj.getChannelState().toString());
			amiEvtResponse.put("ChannelStateDesc", evtObj.getChannelStateDesc());
			amiEvtResponse.put("CallerIDNum", evtObj.getCallerId());
			amiEvtResponse.put("CallerIDName", evtObj.getCallerIdName());
			amiEvtResponse.put("ConnectedLineNum", evtObj.getConnectedLineNum());
			amiEvtResponse.put("ConnectedLineName", evtObj.getConnectedLineName());
			amiEvtResponse.put("Language", evtObj.getLanguage());
			amiEvtResponse.put("AccountCode", evtObj.getAccountCode());
			amiEvtResponse.put("Context", evtObj.getContext());
			amiEvtResponse.put("Exten", evtObj.getExten());
			amiEvtResponse.put("Priority", evtObj.getPriority().toString());
			amiEvtResponse.put("Uniqueid", evtObj.getUniqueId());
			amiEvtResponse.put("Linkedid", evtObj.getLinkedId());

			if (evtObj.getCause() != null) {
			    amiEvtResponse.put("Cause", evtObj.getCause().toString());
			}

			amiEvtResponse.put("Cause-txt", evtObj.getCauseTxt());
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);		    

		}

		if (event instanceof HangupRequestEvent) {

			HangupRequestEvent evtObj = (HangupRequestEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "HangupRequest");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("Channel", evtObj.getChannel());
			amiEvtResponse.put("ChannelState", evtObj.getChannelState().toString());
			amiEvtResponse.put("ChannelStateDesc", evtObj.getChannelStateDesc());
			amiEvtResponse.put("CallerIDNum", evtObj.getCallerId());
			amiEvtResponse.put("CallerIDName", evtObj.getCallerIdName());
			amiEvtResponse.put("ConnectedLineNum", evtObj.getConnectedLineNum());
			amiEvtResponse.put("ConnectedLineName", evtObj.getConnectedLineName());
			amiEvtResponse.put("Language", evtObj.getLanguage());
			amiEvtResponse.put("AccountCode", evtObj.getAccountCode());
			amiEvtResponse.put("Context", evtObj.getContext());
			amiEvtResponse.put("Exten", evtObj.getExten());
			amiEvtResponse.put("Priority", evtObj.getPriority().toString());
			amiEvtResponse.put("Uniqueid", evtObj.getUniqueId());
			amiEvtResponse.put("Linkedid", evtObj.getLinkedId());

			if (evtObj.getCause() != null) {
			    amiEvtResponse.put("Cause", evtObj.getCause().toString());
			}
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);		    

		}


		if (event instanceof NewStateEvent) {

			NewStateEvent evtObj = (NewStateEvent) event;

			Map<String, String> amiEvtResponse = new LinkedHashMap<String, String>();

			amiEvtResponse.put("Event", "NewState");

			amiEvtResponse.put("Privilege", evtObj.getPrivilege());
			amiEvtResponse.put("SystemName", evtObj.getSystemName());

			amiEvtResponse.put("Channel", evtObj.getChannel());
			amiEvtResponse.put("ChannelState", evtObj.getChannelState().toString());
			amiEvtResponse.put("ChannelStateDesc", evtObj.getChannelStateDesc());
			amiEvtResponse.put("CallerIDNum", evtObj.getCallerId());
			amiEvtResponse.put("CallerIDName", evtObj.getCallerIdName());
			amiEvtResponse.put("ConnectedLineNum", evtObj.getConnectedLineNum());
			amiEvtResponse.put("ConnectedLineName", evtObj.getConnectedLineName());
			amiEvtResponse.put("Language", evtObj.getLanguage());
			amiEvtResponse.put("AccountCode", evtObj.getAccountCode());
			amiEvtResponse.put("Context", evtObj.getContext());
			amiEvtResponse.put("Exten", evtObj.getExten());
			amiEvtResponse.put("Priority", evtObj.getPriority().toString());
			amiEvtResponse.put("Uniqueid", evtObj.getUniqueId());
			amiEvtResponse.put("Linkedid", evtObj.getLinkedId());
			
			return new AmiResponseWrapper(AmiResponseBehavior.SEND, amiEvtResponse);		    

		}

		return new AmiResponseWrapper(AmiResponseBehavior.IGNORE, null);
	}

}
