package com.amirouter.server.services;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketAddress;
import java.net.SocketException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.asteriskjava.manager.AuthenticationFailedException;
import org.asteriskjava.manager.TimeoutException;
import org.asteriskjava.manager.event.ManagerEvent;
import org.asteriskjava.manager.event.PeerStatusEvent;
import org.asteriskjava.manager.event.NewStateEvent;
import org.asteriskjava.manager.event.ContactStatusEvent;
import org.asteriskjava.manager.event.HangupEvent;
import org.asteriskjava.manager.event.HangupRequestEvent;
import org.asteriskjava.manager.event.ParkedCallEvent;
import org.asteriskjava.manager.event.UnparkedCallEvent;
import org.asteriskjava.manager.event.ParkedCallTimeOutEvent;
import org.asteriskjava.manager.event.ParkedCallGiveUpEvent;
import org.asteriskjava.manager.event.DialBeginEvent;
import org.asteriskjava.manager.event.HoldEvent;
import org.asteriskjava.manager.event.UnholdEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.amirouter.server.ami.AmiClientContext;
import com.amirouter.server.ami.AmiResponseWrapper;
import com.amirouter.server.asterisk.AsteriskEvtHandler;
import com.amirouter.server.asterisk.AsteriskInfo;
import com.amirouter.server.asterisk.AsteriskProxy;
import com.amirouter.server.util.NodeInfo;
import com.amirouter.server.util.RegisterResult;
import com.amirouter.server.util.ServerConfig;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.dataformat.yaml.YAMLFactory;

/**
 * Server class that will route 'actions' and 'events' between asterisk servers
 * and clients
 */
@Service
public class ConnManagerService {

	private ServerSocket serverEnd = null;
	private ExecutorService threadPool;

	private ConcurrentHashMap<String, AsteriskProxy> addrToAsteriskProxies;

	private Map<String, String> asteriskNodenameToIp = null;

	private ServerConfig srvConfig = null;

	private volatile boolean started;

	Vector<Socket> clientSocketList = null;

	Pattern peerPattern = Pattern.compile(".*-(.*)");

	@Autowired
	AmiProtocolService amiProtocolService;

	/**
	 * Verify that the server config file passed through command argument '-c' exist
	 * 
	 * @param filePathStr the config file path passed through command argument '-c'
	 *                    exist
	 * @return true if file exist and false if not
	 */
	public boolean verifyConfigFile(String filePathStr) {
		File f = new File(filePathStr);
		if (f.exists() && !f.isDirectory()) {
			return true;
		}

		System.err.println(String.format("--> %s file doesn't exist or is a directory", filePathStr));
		return false;
	}

	/**
	 * Print current server in the amirouter cli
	 */
	public void showConfigContent() {
		if (srvConfig != null) {
			System.out.println(srvConfig.toString());
			return;
		}

		System.err.println("--> config is empty");
	}

	/**
	 * Start handling incoming client connection
	 * 
	 * @param configFilePath amirouter config file path
	 */
	public void start(String configFilePath) {
		try {

			addrToAsteriskProxies = new ConcurrentHashMap<>();

			asteriskNodenameToIp = new HashMap<String, String>();

			clientSocketList = new Vector<>();

			if (!verifyConfigFile(configFilePath)) {
				return;
			}

			srvConfig = loadConfig(configFilePath);

			if (srvConfig == null) {
				System.out.println("--> Failed to read config file: " + configFilePath);
				System.exit(0);
			}

			if (!startAsteriskProxies()) {
				System.out.println("--> Failed to initialize remote asterisk servers proxies");
				System.exit(0);
			}

			threadPool = Executors.newCachedThreadPool();

			serverEnd = new ServerSocket();

			SocketAddress address = new InetSocketAddress(srvConfig.getListenHost(), srvConfig.getListenPort());
			serverEnd.bind(address);

			System.out.println(String.format("--> Server listening on host %s | port %d ...", srvConfig.getListenHost(),
					srvConfig.getListenPort()));

			started = true;

			threadPool.submit(new InConnectionHandlerThd());

		} catch (Exception e) {
			System.out.println("--> Server failed to start ");
			e.printStackTrace();
		}
	}

	/**
	 * Tell if amirouter is started.
	 * 
	 * @return true if started else false.
	 */
	public boolean isStarted() {
		return started;
	}

	/**
	 * Stop amirouter
	 */
	public void stop() {
		shutdownServer();
	}

	/**
	 * Close all clients connections
	 */
	private void closeAllClientSocket() {
		Iterator<Socket> iterate = clientSocketList.iterator();

		Socket clientSocketObject;

		while (iterate.hasNext()) {

			try {
				clientSocketObject = iterate.next();

				if (!clientSocketObject.isClosed()) {

					clientSocketObject.close();

				}

			} catch (IOException e) {
				e.printStackTrace();
			}

		}

	}

	/**
	 * Release resources and exist the server program
	 */
	private void shutdownServer() {

		if (!started) {
			return;
		}

		try {
			try {
				serverEnd.close();
				stopAsteriskProxies();

				if (clientSocketList != null) {
					closeAllClientSocket();
				}

			} catch (IOException ex) {
			}
			threadPool.shutdown();
			threadPool.awaitTermination(2, TimeUnit.SECONDS);

		} catch (Exception e) {
			e.printStackTrace();
		}

		started = false;
		threadPool = null;
		clientSocketList = null;

	}

	/**
	 * Incoming client connections handler
	 */
	private class InConnectionHandlerThd implements Runnable {

		@Override
		public void run() {
			try {
				try {

					while (started) {
						Socket socketObject = serverEnd.accept();
						System.out.println("--> New Client connected");

						clientSocketList.add(socketObject);
						threadPool.submit(new ClientConnectionHandler(socketObject));
					}
				} catch (SocketException e) {
					System.out.println("--> Server is stopping ...");
				} finally {
					shutdownServer();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

	}

	/**
	 * Create an server config object given the config file path
	 * 
	 * @param configFilePath the server config file passed through command argument
	 *                       '-c'
	 * @return an object containing all the informations in the config file if valid
	 *         or null if not
	 */
	private ServerConfig loadConfig(String configFilePath) {

		try {

			File file = new File(configFilePath);

			ObjectMapper om = new ObjectMapper(new YAMLFactory());

			ServerConfig tmpAppSrvConfig = om.readValue(file, ServerConfig.class);

			System.out.println("--> App Server Config " + tmpAppSrvConfig.toString());

			return tmpAppSrvConfig;

		} catch (IOException e) {
			e.printStackTrace();
		}

		return null;

	}

	/**
	 * Create and run asterisk proxies by using the connections informations
	 * provided in the config file
	 * 
	 * @return true if success and false if not
	 */
	private boolean startAsteriskProxies() {

		List<AsteriskInfo> asteriskServersList = srvConfig.getAsteriskServers();

		if (asteriskServersList.size() <= 0) {
			return false;
		}

		try {
			ListIterator<AsteriskInfo> it = asteriskServersList.listIterator();
			AsteriskInfo tmpAstSrv;
			AsteriskProxy tmpRemAstProxy;

			while (it.hasNext()) {
				tmpAstSrv = it.next();

				tmpRemAstProxy = new AsteriskProxy(tmpAstSrv, tmpAstSrv);

				tmpRemAstProxy.run();

				addrToAsteriskProxies.put(tmpAstSrv.getHostname(), tmpRemAstProxy);

				asteriskNodenameToIp.put(tmpAstSrv.getNodename(), tmpAstSrv.getHostname());

			}

			return true;

		} catch (IOException e) {
			e.printStackTrace();

		} catch (AuthenticationFailedException e) {
			e.printStackTrace();
		} catch (TimeoutException e) {
			e.printStackTrace();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		return false;
	}

	/**
	 * Stop asterisk proxies
	 */
	private void stopAsteriskProxies() {

		addrToAsteriskProxies.forEach((k, runningAsteriskProxy) -> {
			runningAsteriskProxy.finish();
		});

	}

	/**
	 * Handle single client connection in an isolated thread
	 */
	private class ClientConnectionHandler implements Runnable, AsteriskEvtHandler {
		private Socket socketObject = null;
		private Map<String, RegisterResult> asteriskProxiesRegIds = null;

		private boolean isRegistered = false;

		private PrintWriter toClient;

		private final int MAX_AMI_CMD_LINE = 100;

		// TODO: define version inside ami router config ???
		private final String AMI_ROUTER_ASTERISK_VERSION = "Asterisk Call Manager/X.X.0";
		private final String AMI_DEMILITER = ": ";

		private AmiClientContext amiClientContext;

		/**
		 * Constructor
		 * 
		 * @param socketObject the client socket connection
		 */
		public ClientConnectionHandler(Socket socketObject) {
			this.socketObject = socketObject;

			asteriskProxiesRegIds = new HashMap<String, RegisterResult>();

			amiClientContext = new AmiClientContext(srvConfig, asteriskNodenameToIp, addrToAsteriskProxies);
		}

		/**
		 * The callback triggered when an event is received by the proxied asterisk
		 * server.
		 * 
		 * @param event the event received from the proxied asterisk server.
		 */
		@Override
		public void onRemAsteriskEvt(ManagerEvent event) {
			if (isRegistered) {
				try {

				    System.out.println(String.format("--> detected event: %s", event.getClass()));

				    if (!((event instanceof PeerStatusEvent) || 
					  (event instanceof NewStateEvent) || 
					  (event instanceof ContactStatusEvent) ||
					  (event instanceof HangupEvent) ||
					  (event instanceof HangupRequestEvent) ||
					  (event instanceof ParkedCallEvent) ||
					  (event instanceof UnparkedCallEvent) ||
					  (event instanceof ParkedCallTimeOutEvent) ||
					  (event instanceof ParkedCallGiveUpEvent) ||
					  (event instanceof DialBeginEvent) ||
					  (event instanceof HoldEvent) ||
					  (event instanceof UnholdEvent) 
					  )) {
						System.out.println(String.format("--> ignoring event 00: %s", event.getClass()));
						return;
					}

				    String evtPeer = "NONE";

				    if (event instanceof PeerStatusEvent) {
					evtPeer = ((PeerStatusEvent) event).getPeer();
				    } else if (event instanceof NewStateEvent) {
					evtPeer = ((NewStateEvent) event).getChannel().substring(0,((NewStateEvent) event).getChannel().lastIndexOf("-")); // Like PJSIP/106-DEVEL-00000041
				    } else if (event instanceof ContactStatusEvent) {
					evtPeer = ((ContactStatusEvent) event).getEndpointName();
				    } else if (event instanceof HangupEvent) {
					evtPeer = ((HangupEvent) event).getChannel().substring(0,((HangupEvent) event).getChannel().lastIndexOf("-")); // Like PJSIP/106-DEVEL-00000041
				    } else if (event instanceof HangupRequestEvent) {
					evtPeer = ((HangupRequestEvent) event).getChannel().substring(0,((HangupRequestEvent) event).getChannel().lastIndexOf("-")); // Like PJSIP/106-DEVEL-00000041
				    } else if (event instanceof ParkedCallEvent) {
					evtPeer = ((ParkedCallEvent) event).getParkingSpace()+"-"+((ParkedCallEvent) event).getParkinglot();
				    } else if (event instanceof UnparkedCallEvent) {
					evtPeer = ((UnparkedCallEvent) event).getParkingSpace()+"-"+((UnparkedCallEvent) event).getParkinglot();
				    } else if (event instanceof ParkedCallGiveUpEvent) {
					evtPeer = ((ParkedCallGiveUpEvent) event).getParkingSpace()+"-"+((ParkedCallGiveUpEvent) event).getParkinglot();
				    }

				    System.out.println(String.format("--> Identify peer/extension %s", evtPeer));
					
				    Matcher m = peerPattern.matcher(evtPeer);

					if ((m.matches()) && (m.groupCount() > 0)) {

					    System.out.println(String.format("--> Identified peer/extension"));

						String clientAccountCode = amiProtocolService.getUserService().getAccountCode(amiClientContext);

						if (clientAccountCode.equals(m.group(1).trim())) {
						    System.out.println(String.format("--> routing new '%s' event",event.getClass()));

							AmiResponseWrapper amiEvt = amiProtocolService.handleAmiEvent(event, amiClientContext);

							switch (amiEvt.getResult()) {

							case SEND:
								sendAmiResponse(amiEvt.getAmiResponse());
								break;

							default:
								break;
							}

						} else {
							if ((clientAccountCode == null) || (clientAccountCode.trim().isEmpty())) {
								System.out.println(String.format(
										"--> ignoring peer event (peer=%s) | user not logged in.", m.group(1).trim()));
							} else {
								System.out.println(String.format("--> ignoring non-matching peer event (peer=%s)",
										m.group(1).trim()));
							}

						}
					}

				} catch (Exception e) {
					e.printStackTrace();
				}

			}

		}

		/**
		 * Register to asterisk server proxies
		 */
		private void regToAllKnownAsteriskProxies() {

			for (String key : addrToAsteriskProxies.keySet()) {

				AsteriskProxy tmpAstProxy = addrToAsteriskProxies.get(key);

				asteriskProxiesRegIds.put(key,
						new RegisterResult(
								new NodeInfo(key, tmpAstProxy.getInfo().getNodename(), srvConfig.isUsesipregs()),
								tmpAstProxy.registerToEvent(this)));

			}

			isRegistered = true;
		}

		/**
		 * Unregister to asterisk server proxies
		 */
		private void unregToAllKnownAsteriskProxies() {

			isRegistered = false;

			asteriskProxiesRegIds.forEach((k, v) -> {

				AsteriskProxy tmpAstProxy = addrToAsteriskProxies.get(k);

				tmpAstProxy.unRegisterToEvent(v.getRegId());

			});

		}

		/**
		 * Send an ami response to the client
		 * 
		 * @param amiResponse ami response content
		 */
		private void sendAmiResponse(Map<String, String> amiResponse) {

			for (Map.Entry<String, String> entry : amiResponse.entrySet()) {

				toClient.println(entry.getKey() + AMI_DEMILITER + entry.getValue());

			}

			toClient.println();

		}

		@Override
		public void run() {
			try {
				try {
					String rcvData;

					toClient = new PrintWriter(socketObject.getOutputStream(), true);

					BufferedReader fromClient = new BufferedReader(
							new InputStreamReader(socketObject.getInputStream()));

					boolean keepRunning = true;
					boolean amiCmdComplete = false;
					int amiCmdCpt = 0;
					isRegistered = false;

					String parts[];

					regToAllKnownAsteriskProxies();

					toClient.println(AMI_ROUTER_ASTERISK_VERSION);

					while (keepRunning) {

						rcvData = fromClient.readLine();

						if (rcvData == null) {
							break;
						}

						parts = rcvData.split(AMI_DEMILITER, 2);

						Map<String, String> AmiCmd = new LinkedHashMap<String, String>();

						if (parts.length > 1) {
							AmiCmd.put(parts[0].trim(), parts[1].trim());
						} else {
							AmiCmd.put("", parts[0].trim());
						}

						amiCmdComplete = rcvData.trim().isEmpty();
						amiCmdCpt = 0;

						while ((!amiCmdComplete) && (amiCmdCpt < MAX_AMI_CMD_LINE)) {

							rcvData = fromClient.readLine();

							if (rcvData == null) {
								System.err.println("Warning. read null value. Aborting AMI command scan.");
								break;
							} else if (rcvData.trim().isEmpty()) {
								amiCmdComplete = true;
							} else {
								parts = rcvData.split(AMI_DEMILITER, 2);

								if (parts.length != 2) {
									System.err.println("Warning. Unexpected <Key>: <Value> =>" + rcvData
											+ ". Aborting ami command scan.");
									break;
								}

								if (parts.length > 1) {
									AmiCmd.put(parts[0].trim(), parts[1].trim());
								} else {
									AmiCmd.put("", parts[0].trim());
								}

								amiCmdCpt++;

							}
						}

						if (amiCmdComplete) {

							AmiResponseWrapper amiResp = amiProtocolService.handleAmiActionCmd(AmiCmd,
									amiClientContext);

							switch (amiResp.getResult()) {

							case SEND:
								sendAmiResponse(amiResp.getAmiResponse());
								break;

							case SEND_AND_DISCONNECT:
								keepRunning = false;
								sendAmiResponse(amiResp.getAmiResponse());

								break;

							case DISCONNECT:
								keepRunning = false;
								break;

							default:
								break;
							}

						} else if (amiCmdCpt >= MAX_AMI_CMD_LINE) {
							System.err.println("Warning. Too long AMI command. Ignoring.");
						}

					}

				} catch (SocketException e) {
					System.out.println("--> Client socket closed");
				} catch (java.io.EOFException e) {
					System.out.println("--> Client closed his connection");
				} catch (Exception e) {
					e.printStackTrace();
				} finally {

					unregToAllKnownAsteriskProxies();

					System.err.println("--> Client disconnecting...");

					socketObject.close();

				}

			} catch (java.io.EOFException e) {
				System.err.println("--> Client exited...");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

	}

}
