package com.amirouter.server.services;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.amirouter.server.ami.AmiClientContext;
import com.amirouter.server.model.ExExtension;
import com.amirouter.server.model.MuManageruser;
import com.amirouter.server.repo.AsteriskServerInfosRepository;
import com.amirouter.server.repo.ExtensionRepository;
import com.amirouter.server.repo.UserRepository;

/**
 * User (client) management service
 */
@Service
@Transactional(readOnly = true)
public class UserService {

	@Autowired
	private UserRepository userRepository;

	@Autowired
	private ExtensionRepository extensionRepository;

	@Autowired
	private AsteriskServerInfosRepository asteriskServerInfosRepository;

	/**
	 * Tell if client signed in
	 * 
	 * @param amiClientContext client context
	 * @return true if client is signed in else false
	 */
	public boolean isSignedIn(AmiClientContext amiClientContext) {

		return amiClientContext.getCurrentUser() != null;
	}

	/**
	 * client login handler
	 * 
	 * @param username         client username
	 * @param password         client password
	 * @param amiClientContext client context
	 * @return true is username/password matched successfully in the database and
	 *         false if not.
	 */
	public boolean signIn(String username, String password, AmiClientContext amiClientContext) {
		MuManageruser user = userRepository.findByMuName(username);

		if (user == null) {
			System.err.println(String.format("user : %s doesn't exist!", username));
			amiClientContext.setCurrentUser(null);
			return false;
		}

		if (!user.getMuPassword().equals(password)) {
			System.err.println("user password incorrect");
			amiClientContext.setCurrentUser(null);
			return false;
		}

		amiClientContext.setCurrentUser(user);

		return true;
	}

	/**
	 * Handle client logoff
	 * 
	 * @param amiClientContext client context
	 */
	public void signOut(AmiClientContext amiClientContext) {

		amiClientContext.setCurrentUser(null);

	}

	/**
	 * Validate user in client context
	 * 
	 * @param amiClientContext client
	 * @return true if valid else false
	 */
	private boolean isViable(AmiClientContext amiClientContext) {

		return amiClientContext.getCurrentUser() != null;
	}

	/**
	 * Check that user logged in before subsequent operations
	 * 
	 * @param amiClientContext client context
	 */
	private void checkCurrentUser(AmiClientContext amiClientContext) {
		if (!isViable(amiClientContext)) {
			throw new UnsupportedOperationException("current user is not set");
		}
	}

	/**
	 * Retrieve user account code
	 * 
	 * @param amiClientContext client context
	 * @return user account code
	 */
	public String getAccountCode(AmiClientContext amiClientContext) {

		if (!isViable(amiClientContext)) {
			return "";
		}

		return amiClientContext.getCurrentUser().getTeTenant().getTeCode();
	}

	/**
	 * Retrieve extension tech using extension number
	 * 
	 * @param exNumber         extension number
	 * @param amiClientContext client context
	 * @return the corresponding extension tech
	 */
	public String getExTech(String exNumber, AmiClientContext amiClientContext) {
		checkCurrentUser(amiClientContext);

		ExExtension extension = extensionRepository
				.findByTeTenantAndExNumber(amiClientContext.getCurrentUser().getTeTenant(), exNumber);

		String exTech = null;

		if (extension != null) {
			exTech = extension.getExTech();
		}

		return exTech;
	}

	/**
	 * Retrieve remote asterisk server infos
	 * 
	 * @param usesipregs       amirouter config key. @see ServerConfig
	 * @param exNumber         extension number
	 * @param amiClientContext client context
	 * @return an object representing server asterisk server infos
	 */
	public AsteriskServerInfos getAsteriskServerInfos(Boolean usesipregs, String exNumber,
			AmiClientContext amiClientContext) {

		AsteriskServerInfos asteriskServerInfos = null;

		String exTech = getExTech(exNumber, amiClientContext);

		String queryStr = asteriskServerInfosRepository.buildServerInfosQuery(exTech, usesipregs);

		asteriskServerInfos = asteriskServerInfosRepository.retrieveAsteriskServerInfos(queryStr, exNumber,
				amiClientContext.getCurrentUser().getTeTenant().getTeId());

		return asteriskServerInfos;
	}

}
